﻿/*
	The purpose of this script is to split one Illustrator CS4-file by separating groups from
	the layers to individual documents and then exporting them to separate svg- or png-files
	
	Authors:
	otso.koski@nokia.com
	danielb@openismus.com

*/
/* Version 2.0 adds color-story functionality */
/* Version 1.7 adds a warning dialog when about to export a lot of groups */
/* Version 1.6 added option to export sub-group PNGs into their own folder */
/* Version 1.5 fixes an issue with PNG export, adds # ignore option, converts text for SVG to outlines */
/* Version 1.4 added an option to export only selected groups and ignores hidden items */
/* Version 1.3 added a work around for file size limitations when exporting PNGs */
/* Version 1.2 has fixed the colorspace issue also for sub groups */

/*
	COLOR STORY:
	Any group who's name begins with 'colorx' will be exported in these color variations.
	A spot colour swatch called 'colorx' has to exist in the document and be used
	by the element for this to work.
*/
var COLOR_TABLE = {
	color2: 0x62B700, //Phone
	color3: 0x3D890C, //Ovi Maps
	color4: 0x347708, //Ovi Drive
	color5: 0x306600, //Ovi Journeys

	color6: 0x09A7CC, //Conversations
	color7: 0x007DC9, //Contacts
	color8: 0x0067BC, //Email
	color9: 0x1C51AF, //Ovi Store

	// Media
	color10: 0x6400BC, //Photos
	color11: 0x870CBA, //Videos
	color12: 0xCC09BA, //Music
	color13: 0xE800A1, //Music Store

	// Tools
	color14: 0xEF5500, //Calendar
	color15: 0xEA650A, //Documents
	color16: 0xF77219, //Clock
	color17: 0xFF8500, //Feeds
	color18: 0xED9507, //Calculator
	color19: 0xF2B111  //Notes

//     colorfinder: 0x505050
};

/* Show a warning when exporting more groups than this. */
var WARN_GROUPS = 19;

var mainW = new Window('dialog', 'MeeGo Export Script 2.0');
var itemLimit = 0;
var noAlert = false;
var exportPng = false;
var exportSvg = true;
var exportSgToPng = true;
var selectedOnly = app.activeDocument.activeLayer.hasSelectedArtwork;
var exportSubFolders = true;
var exportCustomColors = true;

mainDialog(mainW);

/*******************************************************************************/
 
function mainDialog(mainW) {
	/* copyright notice */
	mainW.cR = mainW.add('statictext', undefined, 'Copyright © 2011 NOKIA Corporation');
	mainW.cR.alignment = "center";
		
	/* PNG andOr SVG check */
	mainW.add('statictext', undefined, 'Export groups:');
	mainW.formatGrp = mainW.add('group', undefined, 'Execute:');
	mainW.formatGrp.svgCheck = mainW.formatGrp.add('checkbox', undefined, 'SVG top-level groups');
	mainW.formatGrp.pngCheck = mainW.formatGrp.add('checkbox', undefined, 'PNG top-level groups');
	mainW.formatGrp.subPngCheck = mainW.formatGrp.add('checkbox', undefined, 'PNG sub groups');	
	mainW.formatGrp.svgCheck.value = true;
	mainW.formatGrp.subPngCheck.value = true;
	mainW.formatGrp.pngCheck.onClick = function() {
		exportPng = this.value;
	};
	mainW.formatGrp.svgCheck.onClick = function() {
		exportSvg = this.value;
	};
	mainW.formatGrp.subPngCheck.onClick = function() {
		mainW.subfoldersGrp.enabled = this.value;
		exportSgToPng = this.value;
	};

	/* export sub-groups to folder option */
	mainW.subfoldersGrp = mainW.add('group');
	mainW.subfoldersGrp.selectOnlyCheck = mainW.subfoldersGrp.add('checkbox', undefined, 'Export sub groups into sub folders');
	mainW.subfoldersGrp.selectOnlyCheck.value = exportSubFolders;
	mainW.subfoldersGrp.selectOnlyCheck.onClick = function() {
		exportSubFolders = this.value;
	};
	mainW.subfoldersGrp.enabled = mainW.formatGrp.subPngCheck.value;

	mainW.add('statictext', undefined, 'Groups starting with a # will not be exported.');

	/* Selected only toggle */
	mainW.selectGrp = mainW.add('group');
	mainW.selectGrp.selectOnlyCheck = mainW.selectGrp.add('checkbox', undefined, 'Export only selected groups');
	mainW.selectGrp.selectOnlyCheck.value = selectedOnly;
	mainW.selectGrp.selectOnlyCheck.onClick = function() {
		selectedOnly = this.value;
	};

	/* Export custom colours option */
	mainW.customColors = mainW.add('group');
	mainW.customColors.customColorsCheck = mainW.selectGrp.add('checkbox', undefined, 'Export custom colours');
	mainW.customColors.customColorsCheck.value = exportCustomColors;
	mainW.customColors.customColorsCheck.onClick = function() {
		exportCustomColors = this.value;
	};

	/* Commit or cancel */
	mainW.executeGrp = mainW.add('group', undefined, 'Execute:');
	mainW.executeGrp.orientation='row';
	mainW.executeGrp.buildBtn1= mainW.executeGrp.add('button',undefined, 'Cancel', {name:'cancel'});
	mainW.executeGrp.buildBtn2 = mainW.executeGrp.add('button', undefined, 'OK', {name:'ok'});
	mainW.executeGrp.buildBtn1.onClick = initActionCanceled;
	mainW.executeGrp.buildBtn2.onClick = initActionOk;
	mainW.frameLocation = [100, 100];
	mainW.alignChildren='fill';
	mainW.executeGrp.spacing = 50;
	mainW.executeGrp.alignment = "center";
	mainW.show();

	return mainW;
}

function initActionCanceled() {
	mainW.hide();
	noAlert = true;
}

function initActionOk() {
	exportedGroups = 0;
	exportedPngs = 0;
	exportedSvgs = 0;
	ignoredHidden = 0;
	ignoredUnnamed = 0;
	ignoredHash = 0;
	
	mainW.hide();
	
	if (!documents.length) return;
	
	destFolder = Folder.selectDialog( 'Select the folder where you want to save the exported files.');

	if (!destFolder) return;
	
	tmpPNG = File(destFolder + '/tmp.png');
	tmpSVG = File(destFolder + '/tmp.svg');
	
	doExport();

	if (tmpPNG.exists) tmpPNG.remove();
	if (tmpSVG.exists) tmpSVG.remove();

	if (noAlert != true)
		alert('Export Finished' + ' \nAccessed top level groups: ' + exportedGroups + '\nExported SVGs: ' + exportedSvgs + '\nExported PNGs: ' + exportedPngs + '\n\nIgnored hidden groups: ' + ignoredHidden + '\nIgnored unnamed groups: ' + ignoredUnnamed + '\nIgnored # groups: ' + ignoredHash);
}

function checkIsIgnored(group)
{
	if (selectedOnly && !group.selected) {
		return true;
	} else if (group.hidden) {
		ignoredHidden++;
		return true;
	} else if (!group.name) {
		ignoredUnnamed++;
		return true;
	} else if (group.name[0] == '#') {
		ignoredHash++;
		return true;
	}

	return false;
}

function doExport() { 
	//itemLimit = mainW.limitPnl.limitGrp.et.text;
	itemLimit = "";
	var exportListPNG = [];
	var exportListSVG = [];
	var subGroups = new Object();
	var groupCount = 0;
	noAlert = false;

// ************* Access the layers one by one **********************************

	for (i = 0; i < activeDocument.layers.length; i++)
	{
		var layer = activeDocument.layers[i];
		
		if (layer.visible == false) continue;

		if (layer.groupItems.length == 0)
			continue;

		for (k = 0; k < layer.groupItems.length; k++)
		{
			if(itemLimit != "" && exportedItems >= itemLimit)
				break;
			
			var group = layer.groupItems[k];	
			if (checkIsIgnored(group)) continue;
			
			if (exportPng) {
				exportListPNG.push(group);
				groupCount++;
			}
			
			if (exportSvg) {
				exportListSVG.push(group);
				groupCount++;
			}
				
			exportedGroups++;
			
			if (!exportSgToPng) continue;
			
			/* Export sub group PNGs) */
			subGroups[group.name] = [];
			subGroup = subGroups[group.name];

			for (s = 0; s < group.groupItems.length; s++)
			{
				if (checkIsIgnored(group.groupItems[s])) continue;
				subGroup.push(group.groupItems[s]);
				groupCount++;
			}
		}
	}

	if (groupCount > WARN_GROUPS) {
		if (!confirm("About to export " + groupCount + " groups.\nThis might take a long time and your computer will be unresponsive until exporting has finished.\n\nDo you want to proceed?"))
			return;
	}

	for (sg in subGroups) {
		for (eg = 0; eg < subGroups[sg].length; eg++)
			exportGroup(subGroups[sg][eg], ExportType.PNG24, sg);
	}

	for (eg = 0; eg < exportListPNG.length; eg++)
		exportGroup(exportListPNG[eg], ExportType.PNG24);
		
	for (eg = 0; eg < exportListSVG.length; eg++)
		exportGroup(exportListSVG[eg], ExportType.SVG);
}

function exportGroup (group, type, parent) {
	app.activeDocument.selection = null;
	group.selected = true;
	copy();

	var newDoc = app.documents.add(DocumentColorSpace.RGB, group.width, group.height) ;
	newDoc.title = group.name;
	newDoc.layers[0].name = group.name + '-layer' ;
	paste();
	
	// Oddly illustrator does not paste at 0,0, so make sure coordinates are precise.
	newDoc.groupItems[0].top = newDoc.height; // So far the only thing that works correctly.. WTF?
	newDoc.groupItems[0].left = 0.0;
	
	// Ensure that parent folder exists.
	if (parent) {
		parentFolder = new Folder(destFolder + '/' + parent);
		if (!parentFolder.exists) {
			parentFolder.create();
		}
	}

	// Export to temporary file first to work around 64 character filename limitation.
	if (type == ExportType.SVG) {
		// Convert all text to outlines to avoid SVG text issues.
		for (var i = 0; i < newDoc.textFrames.length; i++) {
			newDoc.textFrames[i].createOutline();
		}
		exportFileToSVG(tmpSVG.fullName);
		tmpSVG.copy(destFolder + '/' + group.name + '.svg');
	} else {
		// Export colour story
		if (group.name.substr(0, 6) == 'colorx') {
			if (exportCustomColors) {
				var swatch = newDoc.swatches.getByName("colorx");
				var origColor = swatch.color.spot.color;
				var strippedName = group.name.substring(6, group.name.length);

				for (color in COLOR_TABLE) {
					var newColor = getRGBColorFromHex(COLOR_TABLE[color]);
					newColor = getRGBColorFromHex(COLOR_TABLE[color]);
					swatch.color.spot.color = newColor;
					exportFileToPNG24(tmpPNG.fullName);
		
					colorFolder = new Folder(destFolder + '/' + color);
					if (!colorFolder.exists) {
						colorFolder.create();
					}

					tmpPNG.copy(destFolder + '/' + color + '/' + color + strippedName + '.png');
				}
			}
		} else {
			exportFileToPNG24(tmpPNG.fullName);
			if (parent) {
				tmpPNG.copy(destFolder + '/' + parent + '/' + group.name + '.png');
			} else {
				tmpPNG.copy(destFolder + '/' + group.name + '.png');
			}
		}
	}

	newDoc.close(SaveOptions.DONOTSAVECHANGES);
}

function getRGBColorFromHex(hex) {
	newColor = new RGBColor();
	newColor.red   = (hex & 0xff0000) >> 16;
	newColor.green = (hex & 0x00ff00) >> 8;
	newColor.blue  = hex & 0x0000ff;

	return newColor;
}

function exportFileToSVG (dest) { 
	var exportOptions = new ExportOptionsSVG();
	var type = ExportType.SVG;
	var fileSpec = new File(dest);
	exportOptions.embedRasterImages = true;
	exportOptions.embedAllFonts = false;
	exportOptions.fontSubsetting = SVGFontSubsetting.GLYPHSUSED;
	app.activeDocument.exportFile(fileSpec, type, exportOptions);

	exportedSvgs++;
}

function exportFileToPNG24 (dest) {
	var exportOptions = new ExportOptionsPNG24();
	var type = ExportType.PNG24;
	var fileSpec = new File(dest);
	exportOptions.transparency = true;
	exportOptions.matte = false;
	app.activeDocument.exportFile(fileSpec, type, exportOptions);

	exportedPngs++;
}
		
